@@ -6,7 +6,7 @@ import jpype.imports
 
 from .libs import CLASSPATH
 
-logging.basicConfig(level=logging.DEBUG)
+logging.basicConfig(level=logging.WARN)
 logger = logging.getLogger('tuprolog')
 
 jars = [str(j.resolve()) for j in CLASSPATH.glob('*.jar')]
@@ -14,10 +14,12 @@ jars = [str(j.resolve()) for j in CLASSPATH.glob('*.jar')]
 jpype.startJVM(classpath=jars)
 
 # noinspection PyUnresolvedReferences
-from it.unibo.tuprolog import Info
+from it.unibo import tuprolog as _tuprolog
+
+Info = _tuprolog.Info
 
 JVM_VERSION = '.'.join(map(str, jpype.getJVMVersion()))
 
-logger.debug("Started JVM v" + JVM_VERSION + " with classpath: " + str(jars))
+logger.info("Started JVM v" + JVM_VERSION + " with classpath: " + str(jars))
 
-logger.debug("Using 2P-Kt v" + str(Info.VERSION))
+logger.info("Using 2P-Kt v" + str(Info.VERSION))
