@@ -5,8 +5,7 @@ import jpype.imports
 import it.unibo.tuprolog.theory as _theory
 from typing import Iterable, Union
 from tuprolog.core import Clause
-from tuprolog.pyutils import iterable_or_varargs
-from tuprolog.jvmutils import jiterable
+from ._ktadapt import *
 
 Theory = _theory.Theory
 
